#ifndef HASHTABLE_H
#define HASHTABLE_H

#include <stdint.h>
#include "move.h"
#include "pieces.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Entries in the hash table */
typedef struct {
   uint32_t lock;          /* Part of the hash key lock */
   move_t   move;          /* The best move found for this position */
   eval_t   score_upper;   /* Upper bound for the score for this position */
   eval_t   score_lower;   /* Lower bound for the score for this position */
   int16_t  depth_upper;   /* Remaining depth for the search that produced the upper bound */
   int16_t  depth_lower;   /* Remaining depth for the search that produced the lower bound */
   int16_t  generation;
} hash_table_entry_t;

typedef struct {
   hash_table_entry_t *data;
   size_t number_of_elements;
   size_t write_count;
   uint8_t generation;
} hash_table_t;

hash_table_t *create_hash_table(size_t nelem);
void destroy_hash_table(hash_table_t *table);
void store_table_entry(hash_table_t *table, uint64_t key, int depth_lower, int score_lower, int depth_upper, int score_upper, move_t move);
bool retrieve_table(hash_table_t *table, uint64_t key, int *depth_lower, int *score_lower, int *depth_upper, int *score_upper, move_t *move);
void prefetch_hashtable(hash_table_t *table, uint64_t key);
void prepare_hashtable_search(hash_table_t *table);
int count_unused_table_entries(hash_table_t *table);

#ifdef __cplusplus
}
#endif

#endif

