#ifndef MOVELIST_H
#define MOVELIST_H

#include <stdint.h>
#include <stdlib.h>
#include "assert.h"
#include "move.h"

/* datastructure to hold the number of legal moves in a position */
#define MAX_MOVES    256
typedef struct movelist_t {
   move_t *move;
   int *score;
   int num_moves;
   int cur_move;
   int max_moves;
} movelist_t;

static inline void clear_movelist(movelist_t *movelist)
{
   movelist->cur_move = movelist->num_moves = 0;
}

static inline void push_movelist(movelist_t *movelist, move_t m)
{
   if (movelist->num_moves >= movelist->max_moves) {
      movelist->max_moves += MAX_MOVES;
      movelist->move  = (move_t *)realloc(movelist->move, movelist->max_moves * sizeof *movelist->move);
      movelist->score = (int *)realloc(movelist->score, movelist->max_moves * sizeof *movelist->score);
   }

   movelist->move[movelist->num_moves++] = m;
}

static inline int get_move_score(const movelist_t *movelist)
{
   assert(movelist->cur_move > 0);
   assert(movelist->cur_move-1 < movelist->num_moves);

   return movelist->score[movelist->cur_move-1];
}

static inline void set_move_score(movelist_t *movelist, int s)
{
   assert(movelist->cur_move > 0);
   assert(movelist->cur_move-1 < movelist->num_moves);

   movelist->score[movelist->cur_move-1] = s;
}

static inline void rewind_movelist(movelist_t *movelist)
{
   movelist->cur_move = 0;
}

static inline move_t next_move(movelist_t *movelist)
{
   move_t m = 0;

   if (movelist->cur_move < movelist->num_moves) {
      int nm = movelist->cur_move;
      int sv = movelist->score[nm];
      int n;
      for (n=movelist->cur_move+1; n<movelist->num_moves; n++) {
         if (movelist->score[n] > movelist->score[nm])
            nm = n;
      }
      n = movelist->cur_move;
      m = movelist->move[n]; movelist->move[n] = movelist->move[nm]; movelist->move[nm] = m;
      sv = movelist->score[n]; movelist->score[n] = movelist->score[nm]; movelist->score[nm] = sv;

      m = movelist->move[movelist->cur_move];
      movelist->cur_move++;
   }

   return m;
}

#endif


