#include <string.h>
#include <ctype.h>
#include "move_string.h"
#include "movelist.h"
#include "movegen.h"

char *lan_move_string(const board_t *board, move_t move, char *buffer)
{
   static char static_output_buffer[256];
   char *s = static_output_buffer;
   if (buffer)
      s = buffer;
   s[0] = '\0';
   int n = 0;

   n += snprintf(s, (sizeof static_output_buffer), "%s%s",
         square_str[get_move_from(move)], square_str[get_move_to(move)]);


   if (is_promotion_move(board, move)) {
      int id = promotion_map[get_move_to(move) % MAILBOX_FILES];
      n += snprintf(s+n, (sizeof static_output_buffer)-n, "%c", tolower(piece_info[id].symbol[0]));
   }
   return s;
}

char *short_move_string(board_t *board, move_t move, char *buffer)
{
   static const char *file_names[] = { "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p" };
   static const char *rank_names[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11","12","13","14","15","16" };
   static char static_output_buffer[256];
   movelist_t movelist = { NULL };
   char *s = buffer;
   const char *token = "";
   const char *origin = "";

   if (!s) s = static_output_buffer;

   legal_movegen(&movelist, board);

   if (move == 0) {
      snprintf(s, 15, "(null)");
      goto done;
   }

   int from = get_move_from(move);
   int to = get_move_to(move);
   piece_t id = get_move_piece(board, move);
   const char *fp = "", *tp = "";

   if (is_promotion_move(board, move)) {
      int id = promotion_map[get_move_to(move) % MAILBOX_FILES];
      tp = piece_info[id].symbol;
   }
   fp = piece_info[id].symbol;

   if (board->mailbox[get_move_to(move)] != 0) {
      token = "x";
      if (strlen(fp) == 0) 
         fp = file_names[mailbox_to_file[from]];
   }

   /* Test whether the combination piece/to is unambiguous, otherwise dismbiguate */
   if (board == NULL) goto disambiguous;
   int count = 0;
   int n;
   for (n=0; n<movelist.num_moves; n++) {
      if (id == get_move_piece(board, movelist.move[n]) &&
          to == get_move_to(movelist.move[n])) count++;
   }
   if (count <= 1) goto disambiguous;

   /* Try to disambiguate by file */
   count = 0;
   for (n=0; n<movelist.num_moves; n++) {
      if (get_move_piece(board, move) == get_move_piece(board, movelist.move[n]) &&
            to == get_move_to(movelist.move[n]) &&
            mailbox_to_file[from] == mailbox_to_file[get_move_from(movelist.move[n])])
         count++;
   }
   if (count == 1) {
      origin = file_names[mailbox_to_file[from]];
   } else {
      /* Disambiguate by row */
      origin = rank_names[mailbox_to_rank[from]];
   }
disambiguous:

   snprintf(s, 256, "%s%s%s%s%s", fp, origin, token, square_str[to], tp);
done:
   return s;
}

