#include "board.h"
#include "pieces.h"
#include "score.h"

#define H 1                                  /* A Horizontal step       */
#define V MAILBOX_FILES                      /* A Vertical step         */
#define D (MAILBOX_FILES+1)                  /* A Diagonal step         */
#define A (MAILBOX_FILES-1)                  /* An Anti-diagonal step   */
#define L(h,v) ((h)+(v)*MAILBOX_FILES)       /* A Leap                  */
piece_description_t piece_info[MAX_PIECE_TYPES] = {
   { "Pawn",      "", 'P',   0, "fmWfcF",   80, 0, { { V } } },
   { "Pawn",      "",   0, 'p', "fmWfcF",   80, 0, { {-V } } },
   { "King",     "K", 'K', 'k', "KimDimA",CHECKMATE,0, { { H }, {-H }, { V }, {-V }, { D }, {-D }, { A }, {-A } } },
   { "castle",  NULL,   0,   0,NULL,         0, 0, { { 2*H }, {-2*H }, { 2*V }, {-2*V }, { 2*D }, {-2*D }, { 2*A }, {-2*A } } },
   { "Rook",     "R", 'R', 'r', "R",       450, 3, { { 0, H, 11}, { 0, -H, 11}, { 0, V, 11}, { 0, -V, 11} } },
   { "Bishop",   "C", 'C', 'c', "B",       300, 2, { { 0, D, 11}, { 0, -D, 11}, { 0, A, 11}, { 0, -A, 11} } },
   { "Knight",   "N", 'N', 'n', "N",       275, 2, { { L( 1, 2) }, { L(-1, 2) }, { L( 1,-2) }, { L(-1,-2) },
                                                     { L( 2, 1) }, { L(-2, 1) }, { L( 2,-1) }, { L(-2,-1) } } },
   { "Ferz",     "F", 'F', 'f', "F",       150, 1, { { D }, {-D }, { A }, {-A } } },
   { "Wazir",    "W", 'W', 'w', "W",       150, 1, { { H }, {-H }, { V }, {-V } } },
   { "Man",      "M", 'M', 'm', "K",       250, 2, { { H }, {-H }, { V }, {-V }, { D }, {-D }, { A }, {-A } } },
   { "Elephant", "E", 'E', 'e', "A",       125, 1, { {2*D }, {-2*D }, {2*A }, {-2*A } } },
   { "Dababa",   "D", 'D', 'd', "D",       150, 1, { {2*H }, {-2*H }, {2*V }, {-2*V } } },
   { "Zebra",    "G", 'G', 'g', "Z",       175, 1, { { L( 3, 2) }, { L(-3, 2) }, { L( 3,-2) }, { L(-3,-2) },
                                                     { L( 2, 3) }, { L(-2, 3) }, { L( 2,-3) }, { L(-2,-3) } } },
   { "Camel",    "S", 'S', 's', "C",       175, 1, { { L( 3, 1) }, { L(-3, 1) }, { L( 3,-1) }, { L(-3,-1) },
                                                     { L( 1, 3) }, { L(-1, 3) }, { L( 1,-3) }, { L(-1,-3) } } },
   { "Lion",     "L", 'L', 'l', "HC",      375, 2, { { L( 3, 1) }, { L(-3, 1) }, { L( 3,-1) }, { L(-3,-1) },
                                                     { L( 1, 3) }, { L(-1, 3) }, { L( 1,-3) }, { L(-1,-3) },
                                                     {3*H }, {-3*H }, {3*V }, {-3*V } } }, 
   { "Gryphon",  "A", 'A', 'a', "FyafsF",  900, 6, { { L( 1, 1), V, 11 }, { L( 1, 1), H, 11, 1 },
                                                     { L(-1, 1), V, 11 }, { L(-1, 1),-H, 11, 1 },
                                                     { L( 1,-1),-V, 11 }, { L( 1,-1), H, 11, 1 },
                                                     { L(-1,-1),-V, 11 }, { L(-1,-1),-H, 11, 1 } } },
   { "Unicorn",  "U", 'U', 'u',"NmpafsyafW",700, 6,{ { L( 1, 2), D, 11 }, { L(-1, 2), A, 11 }, { L( 1,-2), -A, 11 }, { L(-1,-2), -D, 11 },
                                                     { L( 2, 1), D, 11 }, { L(-2, 1), A, 11 }, { L( 2,-1), -A, 11 }, { L(-2,-1), -D, 11 } } },
   { "Picket",   "V", 'V', 'v', "yafF" },
   { "Giraffe",  "J", 'J', 'j', "afsafyafF" },
};

const int piece_types = sizeof piece_info / sizeof *piece_info;
#undef H
#undef V
#undef D
#undef A
#undef L

uint32_t attack_mask[512] = { 0 };

/* Whether an attack by a piece of a given type on the "from" square to the
 * "to" square is possible.
 * May return false positives for sliders or slider compounds, so these
 * need to be verified. However, if the result is that no attack is
 * possible, this can be trusted.
 */
bool piece_attack_possible(int id, int from, int to)
{
   int index = 256 + from - to;

   return (attack_mask[index] >> id) & 1;
}

